package ru.dbotthepony.mc.otm.core.util

import org.apache.logging.log4j.LogManager

class TickList {
	private val conditional = ArrayDeque<IConditionalTickable>()
	private val once = ArrayDeque<ITickable>()
	private val always = ArrayList<ITickable>()
	private val alwaysValveTime = ArrayList<ITickable>()

	private val conditionalValveTime = ArrayList<IConditionalTickable>()
	private val onceValveTime = ArrayList<ITickable>()

	private var inTicker = false

	fun add(ticker: IConditionalTickable) {
		if (inTicker) {
			conditionalValveTime.add(ticker)
		} else {
			conditional.addFirst(ticker)
		}
	}

	fun add(ticker: ITickable) {
		if (inTicker) {
			onceValveTime.add(ticker)
		} else {
			once.addFirst(ticker)
		}
	}

	fun always(ticker: ITickable) {
		if (inTicker) {
			alwaysValveTime.add(ticker)
		} else {
			always.add(ticker)
		}
	}

	fun add(ticker: IConditionalTickable, condition: Boolean, reason: String) {
		if (!condition) {
			LOGGER.error("Refusing to add tickable $ticker because we $reason", IllegalStateException(reason))
			return
		}

		return add(ticker)
	}

	fun add(ticker: ITickable, condition: Boolean, reason: String) {
		if (!condition) {
			LOGGER.error("Refusing to add tickable $ticker because we $reason", IllegalStateException(reason))
			return
		}

		return add(ticker)
	}

	fun until(ticker: () -> Boolean) = add(IConditionalTickable.wrap(ticker))
	fun `while`(tickerCondition: () -> Boolean, ticker: () -> Unit) = add(
		IConditionalTickable.wrap(
			tickerCondition,
			ticker
		)
	)

	fun until(ticker: () -> Boolean, condition: Boolean, reason: String) = add(IConditionalTickable.wrap(ticker), condition, reason)
	fun `while`(tickerCondition: () -> Boolean, ticker: () -> Unit, condition: Boolean, reason: String) = add(
		IConditionalTickable.wrap(tickerCondition, ticker), condition, reason)

	fun tick() {
		if (inTicker) {
			throw ConcurrentModificationException("Already ticking")
		}

		inTicker = true
		val iterator = conditional.iterator()

		for (ticker in iterator) {
			if (!ticker.canTick) {
				iterator.remove()
			} else {
				ticker.tick()
			}
		}

		for (ticker in once) {
			ticker.tick()
		}

		once.clear()

		for (ticker in always) {
			ticker.tick()
		}

		always.addAll(alwaysValveTime)
		alwaysValveTime.clear()

		for (ticker in conditionalValveTime) {
			conditional.addFirst(ticker)
		}

		for (ticker in onceValveTime) {
			once.addFirst(ticker)
		}

		conditionalValveTime.clear()
		onceValveTime.clear()

		inTicker = false
	}

	fun clear() {
		conditional.clear()
		once.clear()
	}

	companion object {
		private val LOGGER = LogManager.getLogger()
	}
}
