package ru.dbotthepony.mc.otm.menu.tech

import com.google.common.collect.ImmutableList
import net.minecraft.world.SimpleContainer
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.inventory.Slot
import net.minecraft.world.item.ItemStack
import ru.dbotthepony.mc.otm.block.entity.RedstoneSetting
import ru.dbotthepony.mc.otm.block.entity.tech.EnergyServoBlockEntity
import ru.dbotthepony.mc.otm.capability.energy
import ru.dbotthepony.mc.otm.menu.MatteryMenu
import ru.dbotthepony.mc.otm.menu.MatterySlot
import ru.dbotthepony.mc.otm.menu.input.EnumInputWithFeedback
import ru.dbotthepony.mc.otm.menu.widget.LevelGaugeWidget
import ru.dbotthepony.mc.otm.registry.MMenus

class EnergyServoMenu @JvmOverloads constructor(
	p_38852_: Int,
	inventory: Inventory,
	tile: EnergyServoBlockEntity? = null
) : MatteryMenu(MMenus.ENERGY_SERVO, p_38852_, inventory, tile) {
	val dischargeSlot: MatterySlot
	val chargeSlot: MatterySlot

	val powerGauge = LevelGaugeWidget(this, tile?.energy)

	val redstone = EnumInputWithFeedback(this, RedstoneSetting::class.java)

	init {
		if (tile != null) {
			redstone.with(tile.redstoneControl::redstoneSetting)
		}
	}

	init {
		val container = tile?.container ?: SimpleContainer(2)

		dischargeSlot = object : MatterySlot(container, EnergyServoBlockEntity.SLOT_DISCHARGE) {
			override fun mayPlace(itemStack: ItemStack): Boolean {
				return super.mayPlace(itemStack) && (itemStack.energy?.canExtract() ?: false)
			}
		}

		chargeSlot = object : MatterySlot(container, EnergyServoBlockEntity.SLOT_CHARGE) {
			override fun mayPlace(itemStack: ItemStack): Boolean {
				return super.mayPlace(itemStack) && (itemStack.energy?.canReceive() ?: false)
			}
		}

		addSlot(dischargeSlot)
		addSlot(chargeSlot)
		addInventorySlots()
	}

	override val storageSlots: Collection<Slot> = ImmutableList.of(dischargeSlot, chargeSlot)
}
